package cn.iocoder.yudao.module.sale.controller.admin.channelPriceStepClearance;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.sale.vo.channelPriceStepClearance.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.channelPriceStepClearance.ChannelPriceStepClearanceDO;
import cn.iocoder.yudao.module.sale.convert.channelPriceStepClearance.ChannelPriceStepClearanceConvert;
import cn.iocoder.yudao.module.sale.service.channelPriceStepClearance.ChannelPriceStepClearanceService;

@Validated
@RestController
@Api(tags = "管理后台 - 渠道清关费阶梯价格")
@RequestMapping("/sale/channel-price-step-clearance")
public class ChannelPriceStepClearanceController {

    @Resource
    private ChannelPriceStepClearanceService channelPriceStepClearanceService;

    @PostMapping("/create")
    @ApiOperation("创建渠道清关费阶梯价格")
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:create')")
    public CommonResult<Long> createChannelPriceStepClearance(@Valid @RequestBody ChannelPriceStepClearanceCreateReqVO createReqVO) {
        return success(channelPriceStepClearanceService.createChannelPriceStepClearance(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新渠道清关费阶梯价格")
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:update')")
    public CommonResult<Boolean> updateChannelPriceStepClearance(@Valid @RequestBody ChannelPriceStepClearanceUpdateReqVO updateReqVO) {
        channelPriceStepClearanceService.updateChannelPriceStepClearance(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除渠道清关费阶梯价格")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:delete')")
    public CommonResult<Boolean> deleteChannelPriceStepClearance(@RequestParam("id") Long id) {
        channelPriceStepClearanceService.deleteChannelPriceStepClearance(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得渠道清关费阶梯价格")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:query')")
    public CommonResult<ChannelPriceStepClearanceBackVO> getChannelPriceStepClearance(@RequestParam("id") Long id) {
        ChannelPriceStepClearanceDO channelPriceStepClearance = channelPriceStepClearanceService.getChannelPriceStepClearance(id);
        return success(ChannelPriceStepClearanceConvert.INSTANCE.convert(channelPriceStepClearance));
    }

    @GetMapping("/list")
    @ApiOperation("获得渠道清关费阶梯价格列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:query')")
    public CommonResult<List<ChannelPriceStepClearanceBackVO>> getChannelPriceStepClearanceList(@RequestParam("ids") Collection<Long> ids) {
        List<ChannelPriceStepClearanceDO> list = channelPriceStepClearanceService.getChannelPriceStepClearanceList(ids);
        return success(ChannelPriceStepClearanceConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得渠道清关费阶梯价格分页")
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:query')")
    public CommonResult<PageResult<ChannelPriceStepClearanceBackVO>> getChannelPriceStepClearancePage(@Valid ChannelPriceStepClearanceQueryVO query, PageVO page) {
        PageResult<ChannelPriceStepClearanceDO> pageResult = channelPriceStepClearanceService.getChannelPriceStepClearancePage(query, page);
        return success(ChannelPriceStepClearanceConvert.INSTANCE.convertPage(pageResult));
    }

    // @GetMapping("/export-excel")
    // @ApiOperation("导出渠道清关费阶梯价格 Excel")
    // @PreAuthorize("@ss.hasPermission('sale:channel-price-step-clearance:export')")
    // @OperateLog(type = EXPORT)
    // public void exportChannelPriceStepClearanceExcel(@Valid ChannelPriceStepClearanceQueryVO query,
    //           HttpServletResponse response) throws IOException {
    //     List<ChannelPriceStepClearanceDO> list = channelPriceStepClearanceService.getChannelPriceStepClearanceList(query);
    //     // 导出 Excel
    //     List<ChannelPriceStepClearanceBackVO> datas = ChannelPriceStepClearanceConvert.INSTANCE.convertList(list);
    //     ExcelUtils.write(response, "渠道清关费阶梯价格.xls", "数据", ChannelPriceStepClearanceBackVO.class, datas);
    // }
}
