package cn.iocoder.yudao.module.sale.controller.admin.listener;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.AchievementDetailExcelExportPushEvent;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.service.order.OrderQueryService;
import cn.iocoder.yudao.module.sale.convert.deptTarget.DeptTargetConvert;
import cn.iocoder.yudao.module.sale.dto.MyAchievementExportDto;
import cn.iocoder.yudao.module.sale.service.deptTarget.DeptTargetService;
import cn.iocoder.yudao.module.sale.vo.deptTarget.DeptTargetQueryVO;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;

@Component("AchievementDetailExcelListener")
@AllArgsConstructor
@Slf4j
public class AchievementDetailExcelListener {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private DeptTargetService deptTargetService;

    @EventListener(AchievementDetailExcelExportPushEvent.class)
    public void achievementDetailExcelExportPushEvent(AchievementDetailExcelExportPushEvent event) {
        try {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
            String nowTime = formatter.format(LocalDateTime.now());
            String dir = ueProperties.getTempDir().concat("/achievementDetail/excel/");
            File fileDir = new File(dir);
            if (!fileDir.exists()){
                // 不存在则创建一个目录
                fileDir.mkdirs();
            }
            String fileName = event.getUserId().toString().concat(StrUtil.DASHED).concat(event.getUserType().toString()).concat(StrUtil.DASHED).concat(nowTime).concat("shipment_preload.xlsx");
            DeptTargetQueryVO deptTargetQueryVO = JSONObject.parseObject(event.getRequestParams(), DeptTargetQueryVO.class);
            List<MyAchievementExportDto> list = DeptTargetConvert.INSTANCE.convertExportList(deptTargetService.getExportDetailExcel(deptTargetQueryVO));
            ExcelUtils.write(dir, fileName, "业绩明细", MyAchievementExportDto.class, list);
            // 获取到临时文件
            File file = new File(dir + fileName);
            // 创建FileInputStream对象
            FileInputStream fileInputStream = new FileInputStream(file);
            // 读取文件内容
            byte[] fileBytes = new byte[(int) file.length()];
            fileInputStream.read(fileBytes);
            // 关闭文件流
            fileInputStream.close();
            // 将文件上传到资源服务器
            FileDO fileDO = fileService.createFile(dir, fileName, fileBytes);
            event.setPath(fileDO.getPath());
            event.setFileName(fileDO.getPath());
            event.setUrl(fileDO.getUrl());
            event.setFileId(fileDO.getId());
        } catch (Exception e) {
            log.info("业绩明细导出失败",e);
            e.printStackTrace();
            event.setResult(e.getMessage());
        }
    }
}
