package cn.iocoder.yudao.module.sale.controller.admin.offer;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.sale.vo.offer.*;
import cn.iocoder.yudao.module.sale.vo.offerApproval.OfferApprovalBackInfoVO;
import cn.iocoder.yudao.module.system.api.permission.RoleApi;
import cn.iocoder.yudao.module.system.api.permission.dto.RoleRespDTO;
import cn.iocoder.yudao.module.system.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.system.enums.permission.DataScopeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.*;

import javax.annotation.Resource;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;


import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;

import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.ROLE_NOT_EXISTS;

import cn.iocoder.yudao.module.sale.service.offer.OfferService;

@Api(tags = "管理后台 - 报价单管理")
@RestController
@RequestMapping("/ecw/offer")
@Validated
public class OfferController {

    @Autowired
    private OfferService offerService;
    @Resource
    private RoleApi roleApi;


    @PostMapping("/create")
    @ApiOperation("创建报价单管理")
    @Idempotent(timeout = 2)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:create')")
    public CommonResult<Long> createOffer(@Valid @RequestBody OfferCreateReqVO createReqVO) {
        if (Objects.isNull(createReqVO.getStatus())) {
            createReqVO.setStatus(2);
        }
        return success(offerService.createOffer(createReqVO));
    }


    @PutMapping("/update")
    @ApiOperation("更新报价单管理")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:update')")
    public CommonResult<Boolean> updateOffer(@Valid @RequestBody OfferUpdateReqVO updateReqVO) {
        offerService.updateOffer(updateReqVO);
        return success(true);
    }


    @PutMapping("/update/result")
    @ApiOperation("更新报价单结果")
    @Idempotent(timeout = 10)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:update')")
    public CommonResult<Long> updateOfferResult(@Valid @RequestBody OfferResultVO offerResultVO) {
        return success(offerService.updateOfferResult(offerResultVO));
    }

    @PutMapping("/special/apply")
    @ApiOperation("特价申请")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:update')")
    public CommonResult<Boolean> specialApplyOffer(@Valid @RequestBody OfferSpecialApplyVO specialApplyVO) {
        specialApplyVO.setApplyStatus(1);
        offerService.specialApplyOffer(specialApplyVO);
        return success(true);
    }

    @GetMapping("special/info")
    @ApiOperation("获得报价单特价申请详情")
    @ApiImplicitParam(name = "approveId", value = "审核ID", required = true, example = "1024", dataType = "Long")
//    @PreAuthorize("@ss.hasPermission('ecw:offer-approval:query')")
    public CommonResult<OfferApprovalBackInfoVO> getOfferApproval(@RequestParam("approveId") Long approveId) {
        OfferApprovalBackInfoVO infoVO = offerService.approvalOfferInfo(approveId);
        return success(infoVO);
    }

    @GetMapping("special/info/{offerProdId}")
    @ApiOperation("根据报价单商品ID获得报价单商品的特价申请详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "offerProdId", value = "报价单商品ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "type", value = "类型：1 特价优惠 2 佣金申请", defaultValue = "1", dataType = "Integer"),
            @ApiImplicitParam(name = "isResubmit", value = "是否重新提交获取 true/false", required = false, example = "false", dataType = "Boolean")
    })
    //@PreAuthorize("@ss.hasPermission('ecw:offer-approval:query')")
    public CommonResult<OfferApprovalBackInfoVO> getOfferApprovalByOfferProdId(@PathVariable(value = "offerProdId") Long offerProdId,
                                                                               @RequestParam(value = "type", defaultValue = "1") Integer type,
                                                                               @RequestParam(value = "isResubmit", defaultValue = "false") boolean isResubmit) {
        OfferApprovalBackInfoVO infoVO = offerService.approvalOfferSpecialByOfferProdId(type, offerProdId, isResubmit);
        return success(infoVO);
    }

    @GetMapping("cancel/approval/{offerApprovalId}")
    @ApiOperation("根据审批id取消审批")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "offerApprovalId", value = "审批ID", required = true, example = "1024", dataType = "Long"),
            @ApiImplicitParam(name = "reason", value = "取消原因", required = true, example = "1024", dataType = "String")
    })
    //@PreAuthorize("@ss.hasPermission('ecw:offer-approval:update')")
    public CommonResult<Boolean> cancelOfferSpecialApproval(@PathVariable(value = "offerApprovalId") Long offerApprovalId,
                                                            @RequestParam(value = "reason") String reason) {
        offerService.cancelOfferSpecialApproval(offerApprovalId, reason);
        return success(Boolean.TRUE);
    }


    @DeleteMapping("/delete")
    @ApiOperation("删除报价单管理")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:delete')")
    public CommonResult<Boolean> deleteOffer(@RequestParam("id") Long id) {
        offerService.deleteOffer(id);
        return success(true);
    }


    @DeleteMapping("/cancel")
    @ApiOperation("取消报价单管理")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:update')")
    public CommonResult<Boolean> cancelOffer(@RequestParam("id") Long id) {
        offerService.cancelOffer(id);
        return success(true);
    }

    @DeleteMapping("/recovery")
    @ApiOperation("恢复取消的报价单管理")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:update')")
    public CommonResult<Boolean> recoveryOffer(@RequestParam("id") Long id) {
        offerService.recoveryOffer(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得报价单管理")
    @ApiImplicitParam(name = "offerId", value = "报价单ID", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:offer:query')")
    public CommonResult<OfferRespVO> getOffer(@RequestParam("offerId") Long offerId) {
        OfferRespVO respVO = offerService.info(offerId);
        return success(respVO);
    }


    @GetMapping("/page")
    @ApiOperation("获得报价单管理分页")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:query')")
    public CommonResult<PageResult<OfferRespVO>> getOfferPage(@Valid OfferPageReqVO pageVO) {
        PageResult<OfferRespVO> pageResult = offerService.offerPage(pageVO);
        return success(pageResult);
    }


    @GetMapping("/data/space/page")
    @ApiOperation("获得报价单管理数据权限分页")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:query')")
    public CommonResult<PageResult<OfferRespVO>> getDataSpaceOfferPage(@Valid OfferPageReqVO pageVO) {
        if (Objects.isNull(pageVO)) {
            pageVO = new OfferPageReqVO();
        }
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (Objects.isNull(loginUser)){
            return error(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        List<RoleRespDTO> roleRespDTOS = roleApi.getRoles(loginUser.getRoleIds());
        if (Objects.isNull(roleRespDTOS)){
            return error(ROLE_NOT_EXISTS);
        }
        addDataScopeQuery(pageVO, loginUser, roleRespDTOS);
        PageResult<OfferRespVO> pageResult = offerService.offerPage(pageVO);
        return success(pageResult);
    }

    @GetMapping("/dept/page")
    @ApiOperation("获得部门报价单管理分页")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:query')")
    public CommonResult<PageResult<OfferRespVO>> getDeptOfferPage(@Valid OfferPageReqVO pageVO) {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        pageVO.setDeptId(Objects.isNull(loginUser) || Objects.isNull(loginUser.getDeptId()) ? 0L : loginUser.getDeptId());
        PageResult<OfferRespVO> pageResult = offerService.offerPage(pageVO);
        return success(pageResult);
    }

    @GetMapping("/data/space/export-excel")
    @ApiOperation("数据权限导出报价单管理 Excel")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:export')")
    @OperateLog(type = EXPORT)
    public void exportDataSpaceOfferExcel(@Valid OfferPageReqVO pageVO,
                                     HttpServletResponse response) throws IOException {
        if (Objects.isNull(pageVO)) {
            pageVO = new OfferPageReqVO();
        }
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        if (Objects.isNull(loginUser)){
            throw exception(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        List<RoleRespDTO> roleRespDTOS = roleApi.getRoles(loginUser.getRoleIds());
        if (Objects.isNull(roleRespDTOS)){
            throw exception(ROLE_NOT_EXISTS);
        }
        addDataScopeQuery(pageVO, loginUser, roleRespDTOS);
        offerService.exportOfferExcel(response, pageVO);
    }

    @GetMapping("/dept/export-excel")
    @ApiOperation("导出部门报价单管理 Excel")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:export')")
    @OperateLog(type = EXPORT)
    public void exportDeptOfferExcel(@Valid OfferPageReqVO pageVO,
                                 HttpServletResponse response) throws IOException {
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        pageVO.setDeptId(Objects.isNull(loginUser) || Objects.isNull(loginUser.getDeptId()) ? 0L : loginUser.getDeptId());
        offerService.exportOfferExcel(response, pageVO);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出报价单管理 Excel")
//    @PreAuthorize("@ss.hasPermission('ecw:offer:export')")
    @OperateLog(type = EXPORT)
    public void exportOfferExcel(@Valid OfferPageReqVO pageVO,
                                 HttpServletResponse response) throws IOException {
        offerService.exportOfferExcel(response, pageVO);
    }



    @GetMapping("/check")
    @ApiOperation(value = "检查获取订单关联的报价单", notes = "检查获取订单关联的报价单，存在则返回，不存在则返回空值")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OfferRespVO> getOfferByOrderId(@RequestParam("orderId") Long orderId) {
        OfferRespVO respVO = offerService.getOfferByOrderId(orderId);
        return success(respVO);
    }

    @GetMapping("/select/{orderId}")
    @ApiOperation(value = "获得可以关联的报价单列表下拉框数据", notes = "获取订单关联的报价单列表，当订单已关联报价单时会给错误提示，不能再关联其他报价单")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<OfferRespVO>> getOfferListByOrderId(@PathVariable("orderId") Long orderId) {
        List<OfferRespVO> respVOList = offerService.getOfferListByOrderId(orderId);
        return success(respVOList);
    }

    @GetMapping("/select")
    @ApiOperation(value = "获得可以关联的报价单列表下拉框数据", notes = "获取订单关联的报价单列表，当订单已关联报价单时会给错误提示，不能再关联其他报价单")
    public CommonResult<List<OfferRespVO>> getOfferSelect(@Valid OrderAssociationOfferQueryVO queryVO) {
        // 核查订单关联报价单异常
        queryVO.setUserType(2);
        List<OfferRespVO> respVOList = offerService.getOfferSelect(queryVO);
        return success(respVOList);
    }

    @PutMapping("/association")
    @ApiOperation(value = "订单关联报价单", notes = "订单关联报价单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "offerId", value = "报价单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class),
    })

    public CommonResult<Boolean> associationOfferAndOrder(@RequestParam("offerId") Long offerId, @RequestParam("orderId") Long orderId) {
        offerService.associationOfferAndOrder(offerId, orderId);
        return success(true);
    }


    private static void addDataScopeQuery(OfferPageReqVO pageVO, LoginUser loginUser, List<RoleRespDTO> roleRespDTOS) {
        List<Long> deptIdList = new ArrayList<>();
        boolean isAll = Boolean.FALSE;
        for (RoleRespDTO roleRespDTO : roleRespDTOS) {
            DataScopeEnum dataScopeEnum = DataScopeEnum.valueOf(roleRespDTO.getDataScope());
            switch (dataScopeEnum){
                case ALL:
                    // 全部数据权限，不限制
                    isAll = Boolean.TRUE;
                    break;
                case DEPT_CUSTOM:
                    // 指定部门数据权限
                    if (CollectionUtil.isNotEmpty(roleRespDTO.getDataScopeDeptIds())) {
                        deptIdList.addAll(roleRespDTO.getDataScopeDeptIds());
                    }
                    break;
                case DEPT_ONLY:
                    // 部门数据权限
                    if (Objects.nonNull(loginUser.getDeptId())) {
                        deptIdList.add(loginUser.getDeptId());
                    }
                    break;
                case DEPT_AND_CHILD:
                    // 部门及以下数据权限
                    deptIdList.add(loginUser.getDeptId());
                    break;
            }
        }
        if (!isAll){
            // 非全部数据权限时查询限制条件
            if (CollectionUtil.isEmpty(deptIdList) || deptIdList.size() == 0){
                // 没有部门限制条件时，只查询当前用户自己的
                pageVO.setCreator(String.valueOf(loginUser.getId()));
            }else {
                // 查询限定部门的数据
                pageVO.setDeptIdList(deptIdList);
            }
        }
    }
}
