package cn.iocoder.yudao.module.sale.controller.admin.offer;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import static cn.iocoder.yudao.module.sale.enums.ErrorCodeConstants.OFFER_ID_NOT_NULL;

import cn.iocoder.yudao.module.sale.vo.offer.log.*;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferLogDO;
import cn.iocoder.yudao.module.sale.convert.offer.OfferLogConvert;
import cn.iocoder.yudao.module.sale.service.offer.OfferLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 报价单跟进日志")
@RequestMapping("/ecw/offer-log")
public class OfferLogController {

    @Resource
    private OfferLogService offerLogService;

    @PostMapping("/create")
    @ApiOperation("创建报价单跟进日志")
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:create')")
    public CommonResult<Long> createOfferLog(@Valid @RequestBody OfferLogCreateReqVO createReqVO) {
        // 从此处进来的都是商务洽谈的跟进日志
        createReqVO.setType(2);

        return success(offerLogService.createOfferLog(createReqVO));
    }



    @GetMapping("/get")
    @ApiOperation("获得报价单跟进日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:query')")
    public CommonResult<OfferLogBackVO> getOfferLog(@RequestParam("id") Long id) {
        OfferLogBackVO backVO = offerLogService.getCustomerFollow(id);
        return success(backVO);
    }

    @GetMapping("/list/{offerId}")
    @ApiOperation("获得报价单跟进日志列表")
    @ApiImplicitParam(name = "offerId", value = "报价单ID", required = true, dataType = "Long")
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:query')")
    public CommonResult<List<OfferLogBackVO>> getOfferLogListByOfferId(@PathVariable("offerId") Long offerId) {
        List<OfferLogBackVO> list = offerLogService.getOfferLogListByOfferId(offerId);
        return success(list);
    }

    @GetMapping("/page")
    @ApiOperation("获得报价单跟进日志分页")
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:query')")
    public CommonResult<PageResult<OfferLogBackVO>> getOfferLogPage(@Valid OfferLogQueryVO query, PageVO page) {
        if (Objects.isNull(query) || Objects.isNull(query.getOfferId())){
            return error(OFFER_ID_NOT_NULL);
        }
        PageResult<OfferLogBackVO> pageResult = offerLogService.getOfferLogPage(query, page);
        return success(pageResult);
    }

    @GetMapping("/customer/page")
    @ApiOperation("获得客户的报价跟进日志分页")
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:query')")
    public CommonResult<PageResult<OfferLogBackVO>> getCustomerOfferLogPage(@Valid OfferLogQueryVO query, PageVO page) {
        if (Objects.isNull(query) || Objects.isNull(query.getCustomerId())){
            return error(OFFER_ID_NOT_NULL);
        }
        query.setType(2); // 这里只查询客户的跟进的商务洽谈记录，不查操作日志
        PageResult<OfferLogBackVO> pageResult = offerLogService.getCustomerOfferLogPage(query, page);
        return success(pageResult);
    }
    @GetMapping("/export-excel")
    @ApiOperation("导出报价单跟进日志 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:offer-log:export')")
    @OperateLog(type = EXPORT)
    public void exportOfferLogExcel(@Valid OfferLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<OfferLogDO> list = offerLogService.getOfferLogList(query);
        // 导出 Excel
        List<OfferLogBackVO> datas = OfferLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "报价单跟进日志.xls", "数据", OfferLogBackVO.class, datas);
    }
}
