package cn.iocoder.yudao.module.sale.controller.app.channel;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.product.dal.dataobject.product.ProductAttrDO;
import cn.iocoder.yudao.module.product.service.product.ProductAttrService;
import cn.iocoder.yudao.module.sale.convert.channel.ChannelConvert;
import cn.iocoder.yudao.module.sale.dal.dataobject.channel.ChannelDO;
import cn.iocoder.yudao.module.sale.service.channel.ChannelService;
import cn.iocoder.yudao.module.sale.vo.channel.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web - 渠道查询")
@RestController
@RequestMapping("/ecw/channel")
@Validated
public class AppChannelController {

    @Autowired
    private ChannelService channelService;
    @Autowired
    private WarehouseService warehouseService;
    @Autowired
    private ProductAttrService productAttrService;


    @GetMapping("/select")
    @ApiOperation("获得可用渠道列表查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "countryId", value = "目的地国家id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "warehouseId", value = "目的地仓库id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "cityId", value = "目的地城市id", required = false, dataTypeClass = Long.class),
            @ApiImplicitParam(name = "lineId", value = "线路id", required = false, dataTypeClass = Long.class)
    })
    public CommonResult<List<ChannelRespVO>> selectChannelList(Long countryId, Long cityId, Long warehouseId, Long lineId) {
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(warehouseId)){
                WarehouseDO warehouseDO = warehouseService.getWarehouse(warehouseId);
                if (Objects.nonNull(warehouseDO)){
                    countryId = warehouseDO.getGuojia();
                }
            }
        }
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(cityId)){
                WarehouseDO warehouseDO = warehouseService.getWarehouseByShi(cityId);
                if (Objects.nonNull(warehouseDO)){
                    countryId = warehouseDO.getGuojia();
                }
            }
        }
        if (Objects.isNull(countryId)){
            // 国家id为空则选择其他参数传递
            if (Objects.nonNull(lineId)){
                LogisticsInfoDto logisticsInfoDto = warehouseService.getStartInfoAndDestInfoByLineId(lineId);
                if (Objects.nonNull(logisticsInfoDto)){
                    countryId = logisticsInfoDto.getDestCountryId();
                }
            }
        }
        List<ChannelDO> list = channelService.list(new LambdaQueryWrapper<ChannelDO>()
                .in(Objects.nonNull(countryId), ChannelDO::getCountryId, 0,countryId)
                .eq(ChannelDO::getStatus, 1)
                .orderByAsc(ChannelDO::getSort));
        List<ChannelRespVO> resultList = ChannelConvert.INSTANCE.convertList(list);
        List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
        Map<String, String> idNameMap = productAttrDOList.stream()
                .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
        for (ChannelRespVO channelRespVO : resultList) {
            channelRespVO.setAttrNameList(getAttrNameByIds(channelRespVO.getAttrId(), idNameMap));
        }
        return success(resultList);
    }

    @GetMapping("/all/select")
    @ApiOperation("获得全部的渠道信息列表(供订单详情使用，因为可能有历史订单使用的被禁用的渠道信息)")
    public CommonResult<List<ChannelRespVO>> selectAllChannelList() {
        List<ChannelDO> list = channelService.list(new LambdaQueryWrapper<ChannelDO>()
                .orderByAsc(ChannelDO::getSort));
        List<ChannelRespVO> resultList = ChannelConvert.INSTANCE.convertList(list);
        List<ProductAttrDO> productAttrDOList = productAttrService.getProductAttrList();
        Map<String, String> idNameMap = productAttrDOList.stream()
                .collect(Collectors.toMap(attr -> String.valueOf(attr.getId()), ProductAttrDO::getAttrName));
        for (ChannelRespVO channelRespVO : resultList) {
            channelRespVO.setAttrNameList(getAttrNameByIds(channelRespVO.getAttrId(), idNameMap));
        }
        return success(resultList);
    }

    private List<String> getAttrNameByIds(String attrIds, Map<String, String> idNameMap) {
        if(StringUtils.isNotBlank(attrIds)) {
            String[] attrIdArray = attrIds.split(",");
            List<String> nameList = new ArrayList<>();
            for (String attrId : attrIdArray) {
                //map一定包含数据，否则是数据错误
                nameList.add(idNameMap.get(attrId));
            }
            return nameList;
        }
        return null;
    }

}
