package cn.iocoder.yudao.module.sale.controller.app.offer;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import cn.iocoder.yudao.module.sale.vo.offer.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;


import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "app-web后台 - 报价单管理")
@RestController
@RequestMapping("/offer")
@Validated
public class AppOfferController {

    @Autowired
    private OfferService offerService;


    @GetMapping("/get")
    @ApiOperation("获得报价单管理")
    @ApiImplicitParam(name = "offerId", value = "报价单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OfferRespVO> getOffer(@RequestParam("offerId") Long offerId) {
        OfferRespVO respVO = offerService.info(offerId);
        return success(respVO);
    }

    @GetMapping("/check")
    @ApiOperation(value = "检查获取订单关联的报价单", notes = "检查获取订单关联的报价单，存在则返回，不存在则返回空值")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<OfferRespVO> getOfferByOrderId(@RequestParam("orderId") Long orderId) {
        OfferRespVO respVO = offerService.getOfferByOrderId(orderId);
        return success(respVO);
    }

    @GetMapping("/select/{orderId}")
    @ApiOperation(value = "获得可以关联的报价单列表下拉框数据", notes = "获取订单关联的报价单列表，当订单已关联报价单时会给错误提示，不能再关联其他报价单")
    @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<OfferRespVO>> getOfferListByOrderId(@PathVariable("orderId") Long orderId) {
        List<OfferRespVO> respVOList = offerService.getOfferListByOrderId(orderId);
        return success(respVOList);
    }

    @GetMapping("/select")
    @ApiOperation(value = "获得可以关联的报价单列表下拉框数据", notes = "获取订单关联的报价单列表，当订单已关联报价单时会给错误提示，不能再关联其他报价单")
    public CommonResult<List<OfferRespVO>> getOfferSelect(@Valid OrderAssociationOfferQueryVO queryVO) {
        queryVO.setUserType(1);
        List<OfferRespVO> respVOList = offerService.getOfferSelect(queryVO);
        return success(respVOList);
    }

    @PutMapping("/association")
    @ApiOperation(value = "订单关联报价单", notes = "订单关联报价单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "offerId", value = "报价单ID", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "orderId", value = "订单ID", required = true, example = "1024", dataTypeClass = Long.class),
    })

    public CommonResult<Boolean> associationOfferAndOrder(@RequestParam("offerId") Long offerId, @RequestParam("orderId") Long orderId) {
        offerService.associationOfferAndOrder(offerId, orderId);
        return success(true);
    }

}
